@@ -3,7 +3,7 @@ package Bio::MUST::Core::Seq;
 # CONTRIBUTOR: Catherine COLSON <ccolson@doct.uliege.be>
 # CONTRIBUTOR: Arnaud DI FRANCO <arnaud.difranco@gmail.com>
 # CONTRIBUTOR: Valerian LUPO <valerian.lupo@doct.uliege.be>
-$Bio::MUST::Core::Seq::VERSION = '0.212530';
+$Bio::MUST::Core::Seq::VERSION = '0.212650';
 use Moose;
 use MooseX::SemiAffordanceAccessor;
 use namespace::autoclean;
@@ -164,15 +164,67 @@ sub gapify {
     my $self = shift;
     my $char = shift // '*';                # defaults to gap
 
-    my $regex = $PROTMISS;                  # defaults to protein seq
+    # original version: very general but much too slow on large matrices
+    # my $regex = $PROTMISS;                  # defaults to protein seq
+    #
+    # # in case of DNA ensure correct 'missification' (if applicable)
+    # unless ($self->is_protein) {
+    #     $regex = $DNAMISS;
+    #     $char = 'N' if $char =~ $DNAMISS;
+    # }
+    #
+    # ( my $seq = $self->seq ) =~ s{$regex}{$char}xmsg;
 
-    # in case of DNA ensure correct 'missification' (if applicable)
-    unless ($self->is_protein) {
-        $regex = $DNAMISS;
-        $char = 'N' if $char =~ $DNAMISS;
+    # alternative version: hard-coding $char (if possible) gives a 250% boost
+    # my $seq = $self->seq;
+    # if ($self->is_protein) {
+    #     if    ($char eq '*') {
+    #         $seq =~ s{$PROTMISS}{*}xmsg;
+    #     }
+    #     elsif ($char eq 'X') {
+    #         $seq =~ s{$PROTMISS}{X}xmsg;
+    #     }
+    #     else {
+    #         $seq =~ s{$PROTMISS}{$char}xmsg;
+    #     }
+    # }
+    # else {
+    #     if    ($char eq '*') {
+    #         $seq =~ s{$DNAMISS}{*}xmsg;
+    #     }
+    #     elsif ($char eq 'X') {
+    #         $seq =~ s{$DNAMISS}{N}xmsg;
+    #     }
+    #     else {
+    #         $seq =~ s{$DNAMISS}{$char}xmsg;
+    #     }
+    # }
+
+    # current version: hard-coded tr/// for 900% boost with s/// fall-back
+    # Note: $PROTMISS and $DNAMISS regexes in Constants.pm are ignored!
+    my $seq = $self->seq;
+    if ($self->is_protein) {
+        if    ($char eq '*') {
+            $seq =~ tr{?XxBJOUZbjouz}{*};
+        }
+        elsif ($char eq 'X') {
+            $seq =~ tr{?XxBJOUZbjouz}{X};
+        }
+        else {
+            $seq =~ s{[?XxBJOUZbjouz]}{$char}xmsg;
+        }
+    }
+    else {
+        if    ($char eq '*') {
+            $seq =~ tr{?XxNnBDHKMRSVWYbdhkmrsvwy}{*};
+        }
+        elsif ($char eq 'X') {
+            $seq =~ tr{?XxNnBDHKMRSVWYbdhkmrsvwy}{N};
+        }
+        else {
+            $seq =~ s{[?XxNnBDHKMRSVWYbdhkmrsvwy]}{$char}xmsg;
+        }
     }
-
-    ( my $seq = $self->seq ) =~ s{$regex}{$char}xmsg;
 
     $self->_set_seq($seq);
     return $self;
@@ -400,7 +452,7 @@ Bio::MUST::Core::Seq - Nucleotide or protein sequence
 
 =head1 VERSION
 
-version 0.212530
+version 0.212650
 
 =head1 SYNOPSIS
 
